#import "Converter.h"
#import "BarView.h"
#import "DirDragView.h"
#import "ProcessAgent.h"
#import <stdio.h>
#import <sys/file.h>

@implementation Converter

- (id)init
{
    [super init];
    perl = [@"/usr/bin/perl" retain];
    command = [[NSBundle mainBundle] pathForResource:@"mh2next"
                                              ofType:nil];
    workPath = [@"/tmp" retain];
    isRunning = NO;
    isTerminated = NO;
    [command retain];
    return self;
}

- (void)dealloc
{
    [perl release];
    [command release];
    [workPath release];
    return [super dealloc];
}

- (void)forkThread
{
    NSConnection *connection;

    port1a = [NSPort port];
    port1b = [NSPort port];
    connection = [[NSConnection alloc] initWithReceivePort:port1a
                                                  sendPort:port1b];
    [connection setRootObject:self];

    [NSThread detachNewThreadSelector:@selector(doLaunch:)
                             toTarget:self withObject:nil];
}

- (void)doLaunch:(id)obj
{
    NSMutableArray *args;
    NSConnection *connection;
    NSAutoreleasePool *pool;
    NSProxy *rootProxy;
    pool = [[NSAutoreleasePool alloc] init];

    connection = [NSConnection connectionWithReceivePort:port1b
                                                sendPort:port1a];
    rootProxy = [[connection rootProxy] retain];

    aTask = [[[NSTask alloc] init] autorelease];
    [aTask setCurrentDirectoryPath:workPath];
    [aTask setLaunchPath:perl];
    args = [NSArray arrayWithObjects:
        command,
        [mhView filename],
        [mailView filename],
        nil];
    [aTask setArguments:args];
    [aTask launch];
    [aTask waitUntilExit];
    [(Converter *)rootProxy checkStatus:[aTask terminationStatus]];

    [pool release];
    [NSThread exit];
}

- (void)checkStatus:(int)status
{
    if (status != 0) {
        NSString *message = @"Message99";
        NSBundle *bundle = [NSBundle mainBundle];

        if (status == 1) message = @"Message1";
        else if (status == 2) message = @"Message2";
        else if (status == 3) message = @"Message3";
        else if (status == 4) message = @"Message4";
        else if (isTerminated) {
            message = @"Message5";
            isTerminated = NO;
        }
        NSRunAlertPanel(
                        [bundle localizedStringForKey:@"Alert"
                                                value:@""
                                                table:nil],
                        [bundle localizedStringForKey:message
                                                value:@"mh2next error."
                                                table:nil],
                        [bundle localizedStringForKey:@"OK"
                                                value:@"OK"
                                                table:nil],
                        nil,
                        nil
                        );
    }
    [agent invalidate];
    [agent release];
    [barView report:1.0];
    [button setState:0];
    isRunning = NO;
}

- (void)convert:(id)sender
{
    if (isRunning) {
        if ((aTask != nil) && [aTask isRunning]) {
            [aTask terminate];
            isTerminated = YES;
        }
    } else {
        isRunning = YES;
        [barView report:0.0];
        agent =
            [[[ProcessAgent alloc] initFor:barView
                                      with:[mailView filename]] retain];
        [self forkThread];
    }
}

@end
