#import "DirDragView.h"

@implementation DirDragView

- (id)initWithFrame:(NSRect)theRect
{
    [super initWithFrame:theRect];
    [self setDragTypes:[NSArray arrayWithObject:NSFilenamesPboardType]];
    [self registerForDraggedTypes:dragTypes];
    filename = [@"" retain];
    return self;
}

- (void)dealloc
{
    [self setDragTypes:nil];
    [self setImage:nil];
    [filename release];
    [filenames release];
    [desiredType release];
    return [super dealloc];

}

- (void)awakeFromNib
{
    [[self window] makeKeyAndOrderFront: nil];
}

- (void)drawRect:(NSRect)theRect
{
    NSRect bounds;
    NSSize imageSize;

    bounds = [self bounds];
    if (image) {
        imageSize = [image size];
        imageOrigin.x = (bounds.size.width - imageSize.width) / 2.0;
        imageOrigin.y = (bounds.size.height - imageSize.height) / 2.0;
        [image compositeToPoint:imageOrigin operation:NSCompositeSourceOver];
    }
}

- (void)mouseDown:(NSEvent *) theEvent
{
    NSSize              mouseLoc;

    mouseLoc.width = 0;
    mouseLoc.height = 0;
    if (!(image == nil))
        [self dragImage:image
                     at:imageOrigin
                 offset:mouseLoc
                  event:theEvent
             pasteboard:[NSPasteboard pasteboardWithName:NSDragPboard]
                 source:self
              slideBack:YES];
}

- (void)setDragTypes:(NSArray *)newDragTypes
{
    [dragTypes autorelease];
    dragTypes = [newDragTypes retain];
}

- (void)setImage:(NSImage *)newImage
{
    [image autorelease];
    image = [newImage retain];
    [self setNeedsDisplay:YES];
}

- (id)filename
{
    return filename;
}

- (unsigned int)draggingEntered:(id <NSDraggingInfo >)sender
{
    NSPasteboard       *pasteboard;
    unsigned int        sourceDragMask;

    if ([sender draggingSource] != self) {
        sourceDragMask = [sender draggingSourceOperationMask];
        pasteboard = [sender draggingPasteboard];
        desiredType = [[pasteboard availableTypeFromArray:dragTypes]
            retain];

        if (desiredType) {
            if (sourceDragMask & NSDragOperationGeneric) {
                [self setImage:[sender draggedImage]];
                return NSDragOperationGeneric;
            }
        }
        return NSDragOperationNone;
    }
    return NSDragOperationNone;
}

- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender
{
    NSPasteboard *pasteboard;

    pasteboard = [sender draggingPasteboard];
    [filenames release];
    filenames = [[pasteboard propertyListForType:desiredType] retain];
    return YES;
}

- (void)draggingExited:(id <NSDraggingInfo >)sender
{
    if (([sender draggingSource] != self)) {
        [self setImage:nil];
        [self setNeedsDisplay:YES];
    }
}

- (unsigned int)draggingUpdated:(id <NSDraggingInfo>)sender
{	
    return [self draggingEntered:sender];
}

- (void)concludeDragOperation:(id <NSDraggingInfo>)sender
{
    if (([sender draggingSource] != self)) {
        filename = [filenames objectAtIndex:0];
        [field setStringValue:filename];
    }
}

@end
